#pragma once

#include "../../sf.h"
#include <map>


#include "../systemintegration.h"
#include "../../misc/vector.h"

#include <sf/gui/Touches.h>

namespace sf
{
namespace core
{

struct EventInformation
{
};

struct AndroidTouchSubscribeContext;
typedef int AndroidTouchCallback(
								AndroidTouchSubscribeContext *_context,
					 	 	 	int _touch, float _x, float _y
								);

typedef void AndroidScrollCallback(void * _user_data, int _x, int _y, int _dx, int _dy);

struct AndroidTouchSubscribeContext
{
	void *user_data;
	AndroidTouchCallback *_on_touch_begin;
	AndroidTouchCallback *_on_touch_move;
	AndroidTouchCallback *_on_touch_end;
	AndroidTouchCallback *_on_scroll;
};

class CAndroidApplication : public CSystemIntegration
{
public:
	typedef void ButtonCallback();
	enum ButtonCallbackTypes
	{
		ButtonCallbackMenu = 0,
		ButtonCallbackBack,
		ButtonCallbackSearch,

		ButtonCallbackCount
	};

	CAndroidApplication();
	virtual ~CAndroidApplication();

	virtual bool		Init();
	virtual void		Run();
    virtual void        Stop();
	virtual void		Shutdown();

	void	Pause();
	void	Resume();

	void	SetButtonCallback(ButtonCallbackTypes _type, ButtonCallback* _func);

	void	AddScrollCallback(void * _arg, AndroidScrollCallback * _callback);
	void	RemoveScrollCallback(void * _arg, AndroidScrollCallback * _callback);

	void    OnMenuButton();
	void    OnBackButton();
	void    OnSearchButton();

	void    ShowLink(const char* _link);
	void	SetRenderRect(int _width, int _height);

	// return true if we are single, otherwise false and forces focus to other instance
	// relies on g_Application->GetProductName() to identify this process
    virtual bool		EnsureSingleInstance() { return true; }

    virtual void		DoModal();
    virtual void		StopModal();

    virtual bool		ChangeScreenMode(bool _fullscreen, bool _32bpp, size_t _width, size_t _height);
    virtual bool		GetOriginalDesktopDimentions(size_t &_width, size_t &_height);	
    
    misc::IntVector     GetScreenSize() const;

    // touch handling
    void OnTouchBegin(const gui::Touches& _touches, int _timestamp = 0);
    void OnTouchMove(const gui::Touches& _touches, int _timestamp = 0);
    void OnTouchEnd(const gui::Touches& _touches, int _timestamp = 0);
    void OnScroll(int _pid, float _x, float _y, float _dx, float _dy);
    void OnPinch(float x, float y, float _scale, float _velocity, int _state);
    void OnTap(float x, float y);
    void OnDoubleTap(float x, float y);
	
	// this function should be called inside a message-handling process
    virtual EventInformation& GetCurrentEvent();
  
    virtual void		DefaultWindowProc();

    virtual void		GetWindowClientRect(misc::IntRect &_rc);
    virtual void		AdjustClientRectToWindow(misc::IntRect &_rc);
	
    virtual void		GetDesktopWindowedSpace(misc::IntRect &_rc);
    virtual void		ScreenCoordsIntoClient(misc::IntVector& _pos);
    virtual void		ClientCoordsIntoScreen(misc::IntVector& _pos);

	///////////////////////////////////////////////////////////////////////////////
	// Mouse-related section
    // All functions are dummies
    virtual void		SetMouseCursorPos(const misc::IntVector& _pos);
    virtual void		GetMouseCursorPos(misc::IntVector& _pos);
    virtual void		SetSysCursor(gui::SysCursor _cursor, bool _show_now = true);
    virtual gui::SysCursor	GetSysCursor();
    virtual void		ShowSysCursor(bool _show = true);
    virtual bool		IsSysCursorShown();
    

    void PublicAppDraw();
    
    static void SubscribeToTouches(AndroidTouchSubscribeContext *_context) { m_TouchesCallbackContext = _context; }

    static CAndroidApplication* GetMe();
	
	virtual void EnableSystemGestureRecognizer(int _recognizerType, bool _enable);
    
private:
    enum { MaximumEvents = 4 };

    void MessageCycle();

	EventInformation m_Events[MaximumEvents]; // events stack - one for each message loop
	int	m_ModalDepth;

    bool m_EndModal;
	bool m_Stop;
	bool m_Paused;
    int m_LogicalWidth;
    int m_LogicalHeight;
    int m_ScreenWidth;
    int m_ScreenHeight;
    ButtonCallback* m_ButtonCallbacks[ButtonCallbackCount];
    static AndroidTouchSubscribeContext* m_TouchesCallbackContext;

    typedef std::map<AndroidScrollCallback *, void *> ScrollCallbacksT;
    ScrollCallbacksT m_ScrollCallbacks;
    
    bool m_SystemGesturePinch;
    bool m_SystemGestureTap;
    bool m_SystemGestureDoubleTap;
};

inline misc::IntVector CAndroidApplication::GetScreenSize() const
{
    return misc::IntVector(m_ScreenWidth, m_ScreenHeight);
}

} //core
} //sf
