#pragma once

#include "../../sf.h"

#ifndef __SF_ANDROID
	#error This file should be used only on Android platforms
#endif

#include "../../core/resourcemanager.h"
#include "../../core/timer.h"
#include "../../sound/SoundDeclarationsLoader.h"
#include "../../misc/Array.h"

#include <Loki/singleton.h>
#include "AudioPlayer.h"

namespace sf
{
namespace core
{

class CNativeAudio;

// Особенности работы подсистемы для Android:
// Фиксированное органичение на суммарное количество проигрываемых звуков
	
class CAudioManager
{
public:
	enum Values
	{	InvalidHandle	= 0,
		DefaultValue	= -2,
		UnlimitedRepeat = -1,
		MinVolume		= 0,
		MaxVolume		= 100,
		LeftChannel		= -1,
		RightChannel	= 1,
		DefaultSpeed	= 1,
		MaxSpeed		= 10
	};

	CAudioManager();
	~CAudioManager();

    void    SetAllOnDemand(bool _on_demand) {}

	void	SetGlobalVolume(float _volume);
	float	GetGlobalVolume() const;

	//	Загружает из файла звук.
	void	AddGroup(const IdString &_group); 
	
	void	UnloadAllOnDemandSounds() {}
	void	UnloadAll() {}

	bool	IsSound(const IdString &_id) const;
	bool	IsSound(size_t _id) const;

	//	Начать проигрывать звук
	size_t	Play(const IdString &_id, float _balance = DefaultValue, float _volume = DefaultValue, int _priority = DefaultValue, 
		int _repeat_count = DefaultValue, float _speed = DefaultValue);
	
//	size_t	Play(const IdString &_id, const IdString &_group, float _balance = DefaultValue, float _volume = DefaultValue, 
//		int _priority = DefaultValue, int _repeat_count = DefaultValue, float _speed = DefaultValue);

	void	Pause(const IdString& _group_id, bool _pause);
	bool	Pause(size_t _sound_id, bool _pause);
	void	PauseAll(bool _pause);

	bool	Stop(size_t _id, float _time = 0);
	bool	Stop(const IdString &_id, float _time = 0);
	void	StopAll(float _time = 0);
	void	StopGroup(const IdString &_group_id, float _time = 0);
	
	//	Изменить громкость данного звука (_id) на _volume за время time, через время _shift_time (в миллисекундах)
	//  ChangeVolume - изменить на какую-либо величину (дельта как положительная, так и отрицательная)
	bool	ChangeVolume(const IdString &_id, float _delta_volume, float _time = 0, float _shift_time = 0) { return true; }
	bool	ChangeVolume(size_t _id, float _delta_volume, float _time = 0, float _shift_time = 0) { return true; }
	//  SetVolume - установить указанную абсолютную громкость
	bool	SetVolume(const IdString &_id, float _volume, float _time = 0, float _shift_time = 0) { return true; }
	bool	SetVolume(size_t _id, float _volume, float _time = 0, float _shift_time = 0);
	float	GetVolume(const IdString &_id) const; // оригинальная громкость звука (см sounds.xml)
	float	GetVolume(size_t _id) const; // базовая громкость проигрываемого звука
	float	GetMaxVolume(size_t _id) const;

	void	SetGroupVolume(const IdString &_group, float _volume);
	float	GetGroupVolume(const IdString &_group) const;

	bool	SetPriority(const IdString &_id, int _priority);
	bool	SetPriority(size_t _id, int _priority);
	int		GetPriority(const IdString &_id) const;
	int		GetPriority(size_t _id) const;

	bool	SetRepeatCount(const IdString &_id, int _repeat_count) { return true; }
	bool	SetRepeatCount(size_t _id, int _repeat_count) { return true; }
	int		GetRepeatCount(const IdString &_id) const { return true; }
	int		GetRepeatCount(size_t _id) const { return true; }

	//	Ограничения на одновременное воспроизведение звуков
	bool	SetConcurrentSoundPlayDelay(const IdString &_id, float _delay) { return true; }

	double	GetLength(const IdString &_id) const;
	double	GetLength(size_t _id) const; // return length in milliseconds
	double	GetPosition(size_t _id) const { return 0.0; }
	bool	SetPosition(size_t _id, float _seconds) { return true; }

	//	Удаляет звуки, которые закончили играть
	//	Приводит в действие все Actions всех объектов
	bool	Update();
	size_t	GetPlayingNowSoundsAmount() const;

	core::CGroupTimer* GetParentTimer();

	void	CreateSounds(const sound::CSoundDeclarationsLoader &_from);


	void	NotifyPlayingEnded(void* _avplayer);
private:
	enum { MaximumConcurrentStreams = 32 };
	
	// информация о звуках, доступных для проигрывания
	struct SoundInfo
	{
		bool operator<(const SoundInfo& _right) const { return id < _right.id;};
		IdString id;
		IdString group_template;
        StringA120 path;
		unsigned volume;
		int repeat;
		size_t offset;
		size_t length;
		const void * file_mmap;
		float balance;
		int priority;

		int max_concurrent_play;
		int flags;
		float concurrent_sound_play_delay;
		float speed;
		size_t file_size;
	};
	
	struct PlayingSound
	{
		PlayingSound();
        android::CAudioPlayer	player;
		const SoundInfo *info; // parent information
		unsigned		length; // length of sound in milliseconds
		size_t			playing_id; // uid given to this sound playing
		unsigned		playing_volume; // current base volume for this stream (not adjusted by other variables)
		float			prev_bass_volume; // last value that was given to AVPlayer. it's a kind of double check, to prevent equal values to pass there
		int				repeat_count; // < 0 -> infinite playing
		int				played_count;
		bool			is_dead;
		bool			is_finished;
		
		// действие работает если ненулевая первая переменная
		float			action_volume_change_delta;			// can be with minus sign, calculated as OrigVolume - TargetVolume
		float			action_volume_change_applied_delta; // sum[ d ]
		float			action_volume_change_start_time;	// when action should start
		float			action_volume_change_perform_time;	// how long action should perform

		// действие работает если ненулевая первая переменная
		float			action_stop_start_time;
		int             pause_count;
	};
	
	typedef std::vector<SoundInfo> SoundsInfoT;
	typedef std::vector< std::pair<IdString, float> > GroupVolumeRepositoryT;
	typedef sf::misc::Array<PlayingSound, MaximumConcurrentStreams> StreamsT;
	
	size_t					GetNextId();
	const SoundInfo			*FindSoundInfo(const IdString &_id) const;
	float					CalcResultVolumeForStream(const PlayingSound *_sound) const; // вычисляет результирующую громкость для потока в диапазоне 0..100
	void					ChangeVolumeForStream(PlayingSound *_sound, float _delta, float _time, float _shift_time) const; // устанавливает значения переменных измения громкости для отложенных действий
	void					StopPlayingSound(PlayingSound *_sound, float _in_time);
	// останавливает проигрывание звука путем создания соответствующего действия (оно выполняется в update)
	
	StreamsT				m_Streams;
	SoundsInfoT				m_SoundsInfo;
	GroupVolumeRepositoryT	m_GroupVolumeRepository;
	float					m_Volume; // общий уровень громкоски звуков
	core::CGroupTimer*		m_ParentTimer;
	size_t					m_LastId;
};

typedef Loki::SingletonHolder<CAudioManager, Loki::CreateUsingNew, Loki::NoDestroy> g_AudioManager;

} //core
} //sf
