#include "AudioPlayer.h"

namespace sf
{
namespace core
{
namespace android
{

CAudioPlayer::CAudioPlayer()
	: m_VorbisPlayer(0)
{	
}

CAudioPlayer::CAudioPlayer(const CAudioPlayer & rhs)
	: m_VorbisPlayer(rhs.m_VorbisPlayer)
	, m_JavaPlayer(rhs.m_JavaPlayer)
{
	const_cast<CAudioPlayer &>(rhs).m_VorbisPlayer = 0;
}

CAudioPlayer & CAudioPlayer::operator = (const CAudioPlayer & rhs)
{
	m_VorbisPlayer = rhs.m_VorbisPlayer;
	m_JavaPlayer = rhs.m_JavaPlayer;

	const_cast<CAudioPlayer &>(rhs).m_VorbisPlayer = 0;
}

bool CAudioPlayer::Open(const char * _file, const void * _file_mmap, int _offset, int _length, bool _loop)
{
	int len = strlen(_file);
	if(len < 4)
		return false;
	
	const char * ext = _file  + len - 4;

	if(!strcmp(ext, ".ogg"))
	{
		m_VorbisPlayer = new CVorbisPlayer();
		if(!m_VorbisPlayer->Open(_file_mmap, _length, _loop))
		{
			delete m_VorbisPlayer;
			m_VorbisPlayer = 0;
			return false;
		}
		return true;
	}
	return m_JavaPlayer.Open(_file, _offset, _length, _loop);
}

void CAudioPlayer::Close()
{
	m_VorbisPlayer ? m_VorbisPlayer->Close() : m_JavaPlayer.Close();
}

CAudioPlayer::~CAudioPlayer()
{
	if(m_VorbisPlayer)
		delete m_VorbisPlayer;
}

void CAudioPlayer::Start()
{
	m_VorbisPlayer ? m_VorbisPlayer->Start() : m_JavaPlayer.Start();
}

void CAudioPlayer::Stop()
{
	m_VorbisPlayer ? m_VorbisPlayer->Stop() : m_JavaPlayer.Stop();
}

void CAudioPlayer::Pause()
{
	m_VorbisPlayer ? m_VorbisPlayer->Pause() : m_JavaPlayer.Pause();
}

bool CAudioPlayer::IsPlaying() const
{
	return m_VorbisPlayer ? m_VorbisPlayer->IsPlaying() : m_JavaPlayer.IsPlaying();
}

int CAudioPlayer::GetDuration() const
{
	return m_VorbisPlayer ? m_VorbisPlayer->GetDuration() : m_JavaPlayer.GetDuration();
}

void CAudioPlayer::SetVolume(float _volume)
{
	m_VorbisPlayer ? m_VorbisPlayer->SetVolume(_volume) : m_JavaPlayer.SetVolume(_volume);
}

}
}
}