#include <string.h>
#include <sf/diag/assert.h>
#include "AudioTrack.h"

#define STREAM_MUSIC 3
#define CHANNEL_CONFIGURATION_STEREO 12
#define CHANNEL_CONFIGURATION_MONO 4
#define ENCODING_PCM_8BIT 3
#define ENCODING_PCM_16BIT 2
#define MODE_STREAM 1

namespace sf
{
namespace core
{
namespace android
{

static jclass cAudioTrack = 0;

static jmethodID mAudioTrack;
static jmethodID mGetMinBufferSize;
static jmethodID mPlay;
static jmethodID mPause;
static jmethodID mStop;
static jmethodID mRelease;
static jmethodID mWrite;
static jmethodID mSetVolume;

CAudioTrack::CAudioTrack()
	: m_Track(0)
{
}

CAudioTrack::CAudioTrack(const CAudioTrack & rhs)
	: m_Track(rhs.m_Track)
{
	const_cast<CAudioTrack &>(rhs).m_Track = 0;
}

CAudioTrack & CAudioTrack::operator = (const CAudioTrack & rhs)
{
	m_Track = rhs.m_Track;
	const_cast<CAudioTrack &>(rhs).m_Track = 0;	
	return *this;
}

CAudioTrack::~CAudioTrack()
{
	SFASSERT(m_Track == 0);
}

void CAudioTrack::Open(int _sample_rate, int _channels_count, JNIEnv* _env)
{
	SFASSERT(_channels_count <= 2);

	if (!cAudioTrack)
	{
		cAudioTrack = _env->FindClass("android/media/AudioTrack");
		SFASSERT(cAudioTrack);

		mAudioTrack = _env->GetMethodID(cAudioTrack, "<init>", "(IIIIII)V");
		mGetMinBufferSize = _env->GetStaticMethodID(cAudioTrack,
				"getMinBufferSize", "(III)I");
		mPlay = _env->GetMethodID(cAudioTrack, "play", "()V");
		mStop = _env->GetMethodID(cAudioTrack, "stop", "()V");
		mPause = _env->GetMethodID(cAudioTrack, "pause", "()V");
		mRelease = _env->GetMethodID(cAudioTrack, "release", "()V");
		mWrite = _env->GetMethodID(cAudioTrack, "write", "([BII)I");
		mSetVolume = _env->GetMethodID(cAudioTrack, "setStereoVolume", "(FF)I");
	}

	SFASSERT(!m_Track);

	int channelConfig = _channels_count == 1 ? CHANNEL_CONFIGURATION_MONO : CHANNEL_CONFIGURATION_STEREO;
	int audioFormat = ENCODING_PCM_16BIT;

	int bufferSizeInBytes = _env->CallStaticIntMethod(cAudioTrack,
			mGetMinBufferSize, _sample_rate, channelConfig, audioFormat);

	jobject track = _env->NewObject(cAudioTrack, mAudioTrack, STREAM_MUSIC,
			_sample_rate, channelConfig, audioFormat, bufferSizeInBytes,
			MODE_STREAM);

	m_Track = _env->NewGlobalRef(track);
	_env->DeleteLocalRef(track);
}

void CAudioTrack::Close(JNIEnv* _env)
{
	if (m_Track)
	{
		_env->CallNonvirtualVoidMethod(m_Track, cAudioTrack, mStop);
		_env->CallNonvirtualVoidMethod(m_Track, cAudioTrack, mRelease);
		_env->DeleteGlobalRef(m_Track);
		m_Track = 0;
	}
}

void CAudioTrack::Play(JNIEnv* _env)
{
	_env->CallNonvirtualVoidMethod(m_Track, cAudioTrack, mPlay);
}

void CAudioTrack::Pause(JNIEnv* _env)
{
	_env->CallNonvirtualVoidMethod(m_Track, cAudioTrack, mPause);
}

void CAudioTrack::Stop(JNIEnv* _env)
{
	_env->CallNonvirtualVoidMethod(m_Track, cAudioTrack, mStop);
}

void CAudioTrack::Write(jarray _buffer, int size, JNIEnv* _env)
{
	SFASSERT(m_Track);

	_env->CallNonvirtualIntMethod(m_Track, cAudioTrack, mWrite, _buffer, 0,	size);
}

void CAudioTrack::SetVolume(float _vol, JNIEnv* _env)
{
	SFASSERT(m_Track);
	_env->CallIntMethod(m_Track, mSetVolume, _vol, _vol);
}

}
}
}
