#include "sf/diag/assert.h"
#include "JavaAudioPlayer.h"

namespace sf
{
namespace core
{
namespace android
{

jclass CJavaAudioPlayer::m_JavaMediaManager = 0;
jmethodID CJavaAudioPlayer::m_CreateMusic = 0;
jmethodID CJavaAudioPlayer::m_CreateMusicfromGamePack= 0;
jmethodID CJavaAudioPlayer::m_MPStart = 0;
jmethodID CJavaAudioPlayer::m_MPPause = 0;
jmethodID CJavaAudioPlayer::m_MPStop = 0;
jmethodID CJavaAudioPlayer::m_MPIsPlaying = 0;
jmethodID CJavaAudioPlayer::m_MPSetVolume = 0;
jmethodID CJavaAudioPlayer::m_MPGetDuration = 0;
jmethodID CJavaAudioPlayer::m_MPSetLooping = 0;
jmethodID CJavaAudioPlayer::m_MPRelease = 0;
jmethodID CJavaAudioPlayer::m_MPReset = 0;

void CJavaAudioPlayer::Init()
{
	SFASSERT(!m_CreateMusic); // to prevent reinitialization
	SFASSERT(!m_CreateMusicfromGamePack); // to prevent reinitialization
		
	jclass media_player = JNIGetClass("android/media/MediaPlayer");
	m_JavaMediaManager = JNIGetClass("com/stargaze/sf/MediaManager");
        
	m_CreateMusic = JNIGetStaticMethod(m_JavaMediaManager, "createMusic",
                "(Ljava/lang/String;)Landroid/media/MediaPlayer;");
	m_CreateMusicfromGamePack = JNIGetStaticMethod(m_JavaMediaManager, "createMusicfromGamePack",
                "(II)Landroid/media/MediaPlayer;");
				
	m_MPStart = JNIGetObjectMethod(media_player, "start", "()V");
	m_MPPause = JNIGetObjectMethod(media_player, "pause", "()V");
	m_MPStop = JNIGetObjectMethod(media_player, "stop", "()V");
	m_MPIsPlaying = JNIGetObjectMethod(media_player, "isPlaying", "()Z");
	m_MPGetDuration = JNIGetObjectMethod(media_player, "getDuration", "()I");
	m_MPSetVolume = JNIGetObjectMethod(media_player, "setVolume", "(FF)V");
	m_MPSetLooping = JNIGetObjectMethod(media_player, "setLooping", "(Z)V");
	m_MPRelease = JNIGetObjectMethod(media_player, "release", "()V");
	m_MPReset = JNIGetObjectMethod(media_player, "reset", "()V");
}

CJavaAudioPlayer::CJavaAudioPlayer()
	: m_Player(0)
{
}

CJavaAudioPlayer::CJavaAudioPlayer(const CJavaAudioPlayer & rhs)
	: m_Player(rhs.m_Player)
{
	CJavaAudioPlayer & r = const_cast<CJavaAudioPlayer &>(rhs);
	r.m_Player = 0;
}

CJavaAudioPlayer & CJavaAudioPlayer::operator = (const CJavaAudioPlayer & rhs)
{
	m_Player = rhs.m_Player;
	CJavaAudioPlayer & r = const_cast<CJavaAudioPlayer &>(rhs);
	r.m_Player = 0;
}

CJavaAudioPlayer::~CJavaAudioPlayer()
{
	Close();
}

bool CJavaAudioPlayer::Open(const char * _file, int _offset, int _length, bool _loop)
{
	SFASSERT(!m_Player);

	static bool initiated = false;
	if(!initiated)
	{
		Init();
		initiated = true;
	}

	m_Player = CreateMusic(_file);
	if(!m_Player)
		m_Player = CreateMusicfromGamePack(_offset, _length);

	if(!m_Player)
		return false;

	g_JavaEnv->CallVoidMethod(m_Player, m_MPSetLooping, _loop ? JNI_TRUE : JNI_FALSE);
	return true;
}

void CJavaAudioPlayer::Close()
{
	if(m_Player)
	{
		g_JavaEnv->CallVoidMethod(m_Player, m_MPReset);
        g_JavaEnv->CallVoidMethod(m_Player, m_MPRelease);
        m_Player = 0;
    }
}

void CJavaAudioPlayer::Start()
{
	SFASSERT(m_Player);
	g_JavaEnv->CallVoidMethod(m_Player, m_MPStart);
}

void CJavaAudioPlayer::Stop()
{
	SFASSERT(m_Player);
	g_JavaEnv->CallVoidMethod(m_Player, m_MPStop);
}

void CJavaAudioPlayer::Pause()
{
	SFASSERT(m_Player);
	g_JavaEnv->CallVoidMethod(m_Player, m_MPPause);
}

bool CJavaAudioPlayer::IsPlaying() const
{
	SFASSERT(m_Player);
	return g_JavaEnv->CallBooleanMethod(m_Player, m_MPIsPlaying) == JNI_TRUE;
}

int CJavaAudioPlayer::GetDuration() const
{
	SFASSERT(m_Player);
	return g_JavaEnv->CallIntMethod(m_Player, m_MPGetDuration);
}

void CJavaAudioPlayer::SetVolume(float _vol)
{
	SFASSERT(m_Player);
	g_JavaEnv->CallVoidMethod(m_Player, m_MPSetVolume, _vol, _vol);
}

jobject CJavaAudioPlayer::CreateMusic(const StringA120& _path)
{
	jstring path = g_JavaEnv->NewStringUTF(_path.c_str());
        
	jobject music = g_JavaEnv->CallStaticObjectMethod(
	m_JavaMediaManager, m_CreateMusic, path);
            
	g_JavaEnv->DeleteLocalRef(path);
            
	if (music)
	{
		music = g_JavaEnv->NewGlobalRef(music);
		g_JavaEnv->DeleteLocalRef(music);
	}
            
	return music;
}
	
jobject CJavaAudioPlayer::CreateMusicfromGamePack(size_t _offset, size_t _length)
{

	jobject music = g_JavaEnv->CallStaticObjectMethod(
	m_JavaMediaManager, m_CreateMusicfromGamePack, _offset, _length);
        
	if (music)
	{
		music = g_JavaEnv->NewGlobalRef(music);
		g_JavaEnv->DeleteLocalRef(music);
	}
	return music;
}

}
}
}