#pragma once

#include <libvorbis/ivorbisfile.h>
#include <pthread.h>
#include "AudioTrack.h"

namespace sf
{
namespace core
{
namespace android
{

class CVorbisPlayer
{
public:
	CVorbisPlayer();
	~CVorbisPlayer();

	bool Open(const void * file_mmap, int _length, bool _loop);
	void Close();

	void Start();
	void Stop();
	void Pause();
	bool IsPlaying() const;
	int GetDuration() const;
	void SetVolume(float _volume);
	bool IsValid() const;

private:
	static void * ThreadRoutine(void * _arg);
	void PlayRoutine(JNIEnv * _env);

public:
	size_t ReadMmap(void *ptr, size_t size);
	size_t SeekMmap(ogg_int64_t offset, int whence);
	static size_t ReadFile(void *ptr, size_t size, size_t nmemb, void *datasource);
	static int SeekFile(void *datasource, ogg_int64_t offset, int whence);
	static int CloseFile(void *datasource);
	static long TellFile(void *datasource);

private:
	CAudioTrack m_Track;
	mutable OggVorbis_File m_vf;
	const char * m_FileMap;
	const char * m_CurrentPos;
	const char * m_EndPos;
	volatile bool m_Playing;
	volatile bool m_Pause;
	bool m_Loop;
	pthread_t m_Thread;
	pthread_cond_t m_Cond;
	mutable pthread_mutex_t m_Mutex;

private:
	class AutoLock
	{
	public:
		AutoLock(pthread_mutex_t & _mutex)
			: m_Mutex(_mutex)
		{
			pthread_mutex_lock(&m_Mutex);
		}
		~AutoLock()
		{
			pthread_mutex_unlock(&m_Mutex);
		}
	private:
		pthread_mutex_t & m_Mutex;
	};

};

}
}
}
