#include "../../sf.h"

#include "jni.h"

#include "../../diag/log.h"
#include "../../diag/assert.h"

namespace sf
{
namespace core
{

JavaVM* g_JavaVM = 0;
JNIEnv* g_JavaEnv = 0;
jobject g_GameThread = 0;

extern "C" jint JNI_OnLoad(JavaVM* _vm, void* _)
{
    g_JavaVM = _vm;
    return JNI_VERSION_1_4;
}

jclass JNIGetClass(const char* _class)
{
   jclass cls = g_JavaEnv->FindClass(_class);
    if (!cls) 
    {
        SFLOGA("sf", sf::diag::LogError, "Class not found: '%s'", _class);
        SFASSERT(cls);
    }
    
    return cls;
}

jclass JNIGetClassWithGlobalRef(const char* _class)
{
   jclass cls = static_cast <jclass>(g_JavaEnv->NewGlobalRef(JNIGetClass(_class)));
   if(!cls)
   {
       SFLOGA("sf", sf::diag::LogError, "Can't create new global reference for class %s. Out of memory.", _class);
       SFASSERT(cls);
   }
   SFLOGA("sf", sf::diag::LogTrace, "JNI. Global reference for  %s has been created: %x.", _class, (int)cls);
   return cls;
}


jobject JNIGetNewGlobalRef(jobject _object)
{
	jobject obj = g_JavaEnv->NewGlobalRef(_object);
	if(!obj)
	{
		SFLOGA("sf", sf::diag::LogError, "Can't create new global reference for object %x. Out of memory.", (int)_object);
		SFASSERT(obj);
	}
	SFLOGA("sf", sf::diag::LogTrace, "JNI. Global reference created %x", (int)_object);
	return obj;
}
void JNIReleaseGlobalRef(jobject _object)
{
	g_JavaEnv->DeleteGlobalRef(_object);
	SFLOGA("sf", sf::diag::LogTrace, "JNI. Object global reference  %x has been released.", (int)_object);
}


void JNIReleaseGlobalRef(jclass _class)
{
	g_JavaEnv->DeleteGlobalRef(_class);
	SFLOGA("sf", sf::diag::LogTrace, "JNI. Class global reference %x has been released.", (int)_class);
}

jmethodID JNIGetObjectMethod(jclass _class, const char* _method, const char* _signature)
{
    jmethodID method = g_JavaEnv->GetMethodID(_class, _method, _signature);
    if (!method)
    {
        SFLOGA("sf", sf::diag::LogError, "Method not found: '%s' '%s'", _method, _signature);
        SFASSERT(method);
    }
    
    return method;
}

jmethodID JNIGetStaticMethod(jclass _class, const char* _method, const char* _signature)
{
    jmethodID method = g_JavaEnv->GetStaticMethodID(_class, _method, _signature);
    if (!method)
    {
        SFLOGA("sf", sf::diag::LogError, "Static method not found: '%s' '%s'", _method, _signature);
        SFASSERT(method);
    }
    
    return method;
}

jfieldID JNIGetStaticField(jclass _class, const char* _field, const char* _type)
{
    jfieldID field = g_JavaEnv->GetStaticFieldID(_class, _field, _type);
    if (!field)
    {
        SFLOGA("sf", sf::diag::LogError, "Field not found: '%s' '%s'", _field, _type);
        SFASSERT(field);
    }
    
    return field;
}

}
}
